@@ -24,6 +24,7 @@ class OllamaAPI:
         self.model = model
         self.base_url = "http://localhost:11434/api"
         self.token_count = 0
+        self.max_tokens = 128000  # Max tokens for Mistral-Nemo
 
     def count_tokens(self, text):
         return len(tiktoken.encoding_for_model("gpt-4o").encode(text))
@@ -31,6 +32,7 @@ class OllamaAPI:
     def generate(self, prompt):
         url = f"{self.base_url}/generate"
         full_response = ""
+        remaining_tokens = self.max_tokens - self.count_tokens(prompt)
         data = {"model": self.model, "prompt": prompt, "stream": True}
         response = requests.post(url, json=data, stream=True)
         if response.status_code == 200:
@@ -42,6 +44,9 @@ class OllamaAPI:
                         chunk = json_line.get("response", "")
                         full_response += chunk
                         print(chunk, end="", flush=True)
+                        remaining_tokens -= self.count_tokens(chunk)
+                        if remaining_tokens <= 0 or "^^^end^^^" in full_response:
+                            break
                     except json.JSONDecodeError:
                         print(f"Error decoding JSON: {decoded_line}")
         else:
@@ -55,9 +60,7 @@ class OllamaAPI:
         start_index = full_response.find(start_marker)
         end_index = full_response.find(end_marker)
         if start_index != -1 and end_index != -1:
-            full_response = full_response[
-                start_index + len(start_marker) : end_index
-            ].strip()
+            full_response = full_response[start_index + len(start_marker) : end_index].strip()
 
         self.token_count = self.count_tokens(full_response)
         print(f"Token count: {self.token_count}")
@@ -75,6 +78,7 @@ class OpenAIAPI:
             raise ValueError("OPENAI_API_KEY environment variable is not set")
         openai.api_key = self.api_key
         self.token_count = 0
+        self.max_tokens = 16384  # Max tokens for GPT-4
 
     def count_tokens(self, text):
         return len(tiktoken.encoding_for_model(self.model).encode(text))
@@ -82,29 +86,22 @@ class OpenAIAPI:
     def generate(self, prompt):
         try:
             full_response = ""
-            max_iterations = 5  # Adjust this value as needed
-            continuation_prompt = prompt
+            remaining_tokens = self.max_tokens - self.count_tokens(prompt)
+            response = openai.chat.completions.create(
+                model=self.model,
+                messages=[{"role": "user", "content": prompt}],
+                max_tokens=remaining_tokens,
+                stream=True,
+            )
 
-            for iteration in range(max_iterations):
-                response = openai.chat.completions.create(
-                    model=self.model,
-                    messages=[{"role": "user", "content": continuation_prompt}],
-                    stream=True,
-                )
-
-                chunk_response = ""
-                for chunk in response:
-                    if chunk.choices[0].delta.content:
-                        chunk_text = chunk.choices[0].delta.content
-                        chunk_response += chunk_text
-                        print(chunk_text, end="", flush=True)
-
-                full_response += chunk_response
-
-                if "^^^end^^^" in full_response:
-                    break
-
-                continuation_prompt = f"Continue from where you left off. Previous response: {chunk_response}."
+            for chunk in response:
+                if chunk.choices[0].delta.content:
+                    chunk_text = chunk.choices[0].delta.content
+                    full_response += chunk_text
+                    print(chunk_text, end="", flush=True)
+                    remaining_tokens -= self.count_tokens(chunk_text)
+                    if remaining_tokens <= 0 or "^^^end^^^" in full_response:
+                        break
 
             print()  # Print a newline at the end
 
@@ -114,9 +111,7 @@ class OpenAIAPI:
             start_index = full_response.find(start_marker)
             end_index = full_response.find(end_marker)
             if start_index != -1 and end_index != -1:
-                full_response = full_response[
-                    start_index + len(start_marker) : end_index
-                ].strip()
+                full_response = full_response[start_index + len(start_marker) : end_index].strip()
 
             self.token_count = self.count_tokens(full_response)
             print(f"Token count: {self.token_count}")
@@ -136,6 +131,7 @@ class ClaudeAPI:
             raise ValueError("ANTHROPIC_API_KEY environment variable is not set")
         self.client = Anthropic(api_key=self.api_key)
         self.token_count = 0
+        self.max_tokens = 8192  # Max tokens for Claude
 
     def count_tokens(self, text):
         return len(tiktoken.encoding_for_model("gpt-4o").encode(text))
@@ -143,30 +139,22 @@ class ClaudeAPI:
     def generate(self, prompt):
         try:
             full_response = ""
-            max_iterations = 5  # Adjust this value as needed
-            continuation_prompt = prompt
+            remaining_tokens = self.max_tokens - self.count_tokens(prompt)
+            response = self.client.messages.create(
+                model=self.model,
+                messages=[{"role": "user", "content": prompt}],
+                max_tokens=remaining_tokens,
+                stream=True,
+            )
 
-            for iteration in range(max_iterations):
-                response = self.client.messages.create(
-                    model=self.model,
-                    messages=[{"role": "user", "content": continuation_prompt}],
-                    stream=True,
-                    max_tokens=1000,
-                )
-
-                chunk_response = ""
-                for completion in response:
-                    if completion.type == "content_block_delta":
-                        chunk_text = completion.delta.text
-                        chunk_response += chunk_text
-                        print(chunk_text, end="", flush=True)
-
-                full_response += chunk_response
-
-                if "^^^end^^^" in full_response:
-                    break
-
-                continuation_prompt = f"Continue from where you left off. Previous response: {chunk_response}."
+            for completion in response:
+                if completion.type == "content_block_delta":
+                    chunk_text = completion.delta.text
+                    full_response += chunk_text
+                    print(chunk_text, end="", flush=True)
+                    remaining_tokens -= self.count_tokens(chunk_text)
+                    if remaining_tokens <= 0 or "^^^end^^^" in full_response:
+                        break
 
             print()  # Print a newline at the end
 
@@ -176,9 +164,7 @@ class ClaudeAPI:
             start_index = full_response.find(start_marker)
             end_index = full_response.find(end_marker)
             if start_index != -1 and end_index != -1:
-                full_response = full_response[
-                    start_index + len(start_marker) : end_index
-                ].strip()
+                full_response = full_response[start_index + len(start_marker) : end_index].strip()
 
             self.token_count = self.count_tokens(full_response)
             print(f"Token count: {self.token_count}")
@@ -206,6 +192,7 @@ class NemoAgent:
         self.previous_suggestions = set()
         self.token_counts = {}
         self.reference_material = ""
+        self.previous_prompt = ""
 
     def count_tokens(self, text):
         encoding = tiktoken.encoding_for_model("gpt-3.5-turbo")
@@ -445,17 +432,11 @@ class NemoAgent:
                     ***uv_end***
                 3. IMPORTANT: Do not add any code comments to the files.
                 4. IMPORTANT: Always follow PEP8 style guide, follow best practices for Python, use snake_case naming, and provide meaningful docstrings.
-                5. IMPORTANT: Do not redefine built-in functions or use reserved keywords as variable names.
-                6. CRITICAL: Your response should ONLY contain the code blocks and the pip dependencies required for both the test and code files. Do not include any additional information.
-                7. CRITICAL: Only create 1 file for the python code: main.py
-                8. CRITICAL: Only create 1 file for the python tests: tests/test_main.py
-                9. CRITICAL: Create a main method to run the app in main.py and if a web app run the app on port 8080.
-                10. IMPORTANT: Only use pytest fixtures for Flask and FastAPI servers.
-                11. IMPORTANT: Always pytest parameterize tests for different cases.
-                12. CRITICAL: Always use `import main` to import the main.py file in the test file.
-                13. IMPORTANT: Only mock external services or APIs in tests.
-                14. CRITICAL: Enclose your entire response between ^^^start^^^ and ^^^end^^^ markers.
-                15. IMPORTANT: Use the reference material provided to guide your implementation including the required dependencies.
+                5. CRITICAL: Your response should ONLY contain the code blocks and the pip dependencies required for both the test and code files. Do not include any additional information.
+                6. CRITICAL: Create a main method to run the app in main.py and if a web app run the app on port 8080.
+
+                7. CRITICAL: Enclose your entire response between ^^^start^^^ and ^^^end^^^ markers.
+                8. IMPORTANT: Use the reference material provided to guide your implementation including the required dependencies.
             Working directory: {self.pwd}
             Reference material: {self.reference_material}
             """
@@ -511,9 +492,15 @@ class NemoAgent:
 
     def get_response(self, prompt):
         try:
-            response = self.llm.generate(prompt)
+            full_prompt = (
+                f"{self.previous_prompt}\n\n{prompt}"
+                if self.previous_prompt
+                else prompt
+            )
+            response = self.llm.generate(full_prompt)
             prompt_key = prompt[:50]  # Use first 50 characters as a key
             self.token_counts[prompt_key] = self.llm.token_count
+            self.previous_prompt = full_prompt
             return response
         except Exception as e:
             self.logger.error(f"Error getting response from {self.provider}: {str(e)}")
@@ -567,7 +554,7 @@ class NemoAgent:
             score_match = re.search(pattern, complexipy_output, re.DOTALL)
             print(score_match)
             print(complexipy_output)
-            complexipy_score = int(score_match.group(1)) if score_match else None
+            complexipy_score = int(score_match.group(1)) if score_match else 0
 
             print(f"Pylint score for {file_path}: {pylint_score}/10")
             print(f"Complexipy score for {file_path}: {complexipy_score}")
@@ -596,9 +583,6 @@ class NemoAgent:
 
     def improve_test_file(self, test_output):
         prompt = f"""
-        The current test file needs minor improvements. Please analyze the test output and suggest small, specific changes to fix any issues in the test file.
-        Do not modify the main implementation file, only suggest minimal improvements to the tests but write out the full test file content.
-
         Test output:
         {test_output}
 
@@ -606,11 +590,8 @@ class NemoAgent:
 
         Provide specific, minimal code changes to improve the test file, addressing only the failing tests or obvious issues.
         Follow these rules strictly:
-        1. CRITICAL: Only suggest changes to the test file (tests/test_main.py)
-        2. Do not change the code file in main.py
-        3. Focus on fixing failing tests or obvious errors
-        4. Do not rewrite entire test functions unless absolutely necessary
-        5. CRITICAL: Use the following code block format for specifying file content:
+        1. CRITICAL: Only suggest changes to the test file.
+        2. CRITICAL: Use the following code block format for specifying file content:
             For test files, use:
             <<<tests/test_main.py>>>
             # Test file content here
@@ -620,13 +601,8 @@ class NemoAgent:
             ***uv_start***
             package_name, package_name, package_name
             ***uv_end***
-        6. CRITICAL: Do not explain the task only implement the required functionality in the code blocks.
-        7. IMPORTANT: Only use pytest fixtures for Flask and FastAPI servers.
-        8. IMPORTANT: Always pytest parameterize tests for different cases.
-        9. CRITICAL: Always use `import main` to import the main.py file in the test file.
-        10. IMPORTANT: Only mock external services or APIs in tests.
-        11. CRITICAL: Enclose your entire response between ^^^start^^^ and ^^^end^^^ markers.
-        12. CRITICAL: Your response should ONLY contain the code blocks and the pip dependencies required for both the test and code files. Do not include any additional information.
+        3. CRITICAL: Enclose your entire response between ^^^start^^^ and ^^^end^^^ markers.
+        4. CRITICAL: Your response should ONLY contain the code blocks and the pip dependencies required for both the test and code files. Do not include any additional information.
         Working directory: {self.pwd}
         """
         proposed_improvements = self.get_response(prompt)
@@ -687,8 +663,7 @@ class NemoAgent:
         Review the proposed improvements: {proposed_improvements} and confirm if it correctly addresses the original task: {self.task}
         If the implementation is correct or mostly correct, respond with 'VALID'.
         If the implementation is completely unrelated or fundamentally flawed, respond with 'INVALID'.
-        Do not provide any additional information or explanations.
-        IMPORTANT: Enclose your entire response between ^^^start^^^ and ^^^end^^^ markers.
+        Do not provide any additional information or explanations beyond 'VALID' or 'INVALID'.
         """
         response = self.get_response(prompt)
 
